 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Collection Element                      *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/


	/**
	 * Class: thyCollection
	 *
	 * This is the basic ThyAPI Collection Element. With it, one can hold a 
	 * collection of any Element.
	 *
	 */
	function thyCollection(name, contents)
	{
		this.thyBase = thyBase;
		this.thyBase();

		this.name = name;
		this.named = {};
		this.indexed = [];
		this.indexCount = 0;
		this.defaultName = 'Element';
		if (contents != null) this.populate(contents);
	}

	p = dynapi.setPrototype('thyCollection', 'thyBase');

	/**
	 * Method: isFromClass
	 *
	 *	Checks if the object is from the specified class type or from a class
	 *	derivate from that.
	 *
	 * Parameter:
	 *
	 *	className - The name of the class to be checked
	 *
	 * Returns:
	 *
	 *	Boolean
	 *
	 */
	p.isFromClass = function (className)
	{
		return this._className == className || this[className] ? true : false;
	}

	/**
	 * Method: clear
	 *
	 *	Clears collection's values. *DEPRECTED*
	 *
	 *	Use cleanUp or reset
	 *
	 */
	p.clear = function ()
	{
		var i;

		for (i=this.indexed.length-1; i>=0; i--)
		{
			this.remove(this.indexed[i]);
		}
	}

	/**
	 * Method: cleanUp
	 *
	 *	Removes all elements from collection
	 *
	 */
	p.cleanUp = function ()
	{
		var i;

		for (i=this.indexed.length-1; i>=0; i--)
		{
			this.remove(this.indexed[i]);
		}
	}

	/**
	 * Method: reset
	 *
	 *	Removes all elements from collection.
	 *
	 *	Now it is actually just an alias to cleanUp
	 *
	 */
	p.reset = p.cleanUp;
	
	/**
	 * Method: populate
	 *
	 *	Populates a collection with the values of an array.
	 */
	p.populate = function (data)
	{
		if (data == null) throw({result: 'Invalid Data', location: 'thyCollection.populate'});

		if (typeof(data) != 'object') data[this.defaultName+this.indexCount++] = data;
		
		var i, other;
		for (i in data)
		{
			/*if (typeof(data[i]) == 'object')
			{
				other = new thyCollection();
				other.populate(data[i]);
			}
			else*/ other = data[i]
			
			this.add(other, i);
		}
	}

	/**
	 * Method: sweepOut
	 *
	 *	Returns an object with the contents of this thyCollection
	 */
	p.sweepOut = function()
	{
		return this.getNamedElements();
	}

	/**
	 * Method: setDefaultName
	 *
	 *	Sets a default name for elements that are passed without name. 
	 *
	 *	The name given is the default name plus an internal counter.
	 */
	p.setDefaultName = function (name)
	{
		this.defaultName = name;
	}
	
	/**
	 * Method: add
	 *
	 *	Adds a new element into the collection
	 */
	p.add = function (elm, name, index)
	{
		if (name && this.named[name] && 
		   (this.named[name] == elm || (this.getElementIndex(elm) == index && index != null))) return;

		if (this.named[name])
		{
			var oldIndex = this.getElementIndex(this.named[name]);
			this.indexed[oldIndex] = elm;
			this.named[name] = elm;
			return oldIndex;
		}
		
		if (name) this.named[name] = elm;
		else this.named[this.defaultName+this.indexCount++] = elm;

		this.indexed.push(elm);
		if (index != null)
		{
			return this.changeElementIndex({elm: elm, index: index});
		}

		return this.indexed.length-1;
	}

	/**
	 * Method: concat
	 *
	 *	Concatenate this collection with the one passed by parameter.
	 *
	 * Parameter:
	 *
	 *	collection - Another <thyCollection>
	 *
	 */
	p.concat = function(collection)
	{
		var i,named = collection.getNamedElements();

		for (i in named)
		{
			if (this.named[i])
			{
				this.add(this.named[i],i)
				continue;
			}
			
			this.named[i] = named[i];
			this.indexed.push(named[i]);
		}
	}

	/**
	 * Method: removeByName
	 *
	 *	Removes an element of the collection
	 *
	 * Parameters:
	 *
	 *	name - the element name
	 */
	p.removeByName = function (name)
	{
		if (name == null) return;

		if (this.named[name] == null) return;
		
		var elm = this.named[name];
		return this.remove(elm, name);
	}

	/**
	 * Method: removeByIndex
	 *
	 *	Removes an element of the collection
	 *
	 * Parameters:
	 *
	 *	index - the element index
	 */
	p.removeByIndex = function (index)
	{
		if (index == null || typeof(index) != 'number') return;
		
		if (this.indexed[index] == null) return;

		var elm = this.indexed[index];
		return this.remove(elm, null, index);
	}

	/**
	 * Method: remove
	 *
	 *	Removes an element of the collection
	 *
	 * Parameters:
	 *
	 *	elm - the element to be removed
	 */
	p.remove = function (elm, name, index)
	{
		if (name == null && (name = this.getElementName(elm)) == null) return;
		if (index == null && (index = this.getElementIndex(elm)) == null) return;

		var ret = this.named[name];

		this.named[name] = '';
		delete this.named[name];
		var start = this.indexed.slice(0, index);
		var end   = this.indexed.slice(index+1);
		this.indexed = start.concat(end);

		return ret;
	}

	/**
	 * Method: getElementName
	 *
	 *	Gets the name of an element
	 *
	 * Parameters:
	 *
	 *	elm - the element in question
	 */
	p.getElementName = function (elm)
	{	
		var i;
		for (i in this.named)
		{
			if (this.named[i] == elm) return i;
		}
	}

	/**
	 * Method: getElementIndex
	 *
	 *	Gets the index of an element
	 *
	 * Parameters:
	 *
	 *	elm - the element in question
	 */
	p.getElementIndex = function (elm)
	{
		var i;
		for (i=0; i<this.indexed.length; i++)
		{
			if (this.indexed[i] == elm) return i;
		}
	}

	/**
	 * Method: getNamedElements
	 *
	 *	Returns a javascript object with the elements indexed by their names
	 */
	p.getNamedElements = function()
	{
		var i,ret = {};
		
		for (i in this.named)
		{
			ret[i] = this.named[i];
		}
		
		return ret;
	}

	/**
	 * Method: getIndexedElements
	 *
	 *	Returns a javascript array with the elements indexed by their ids used
	 *	internally
	 */
	p.getIndexedElements = function()
	{
		var i,ret = [];

		for (i=0; i<this.indexed.length; i++)
		{
			ret.push(this.indexed[i]);
		}
		
		return ret;
	}

	/**
	 * Method: getElementsCount
	 *
	 *	Returns the number of elements this collection has.
	 */
	p.getElementsCount = function()
	{
		return this.indexed.length;
	}

	/**
	 * Method: length
	 *
	 *	Returns the number of elements this collection has. (alias of getElementsCount)
	 */
	p.length = p.getElementsCount;

	
	/**
	 * Method: getElementByIndex
	 *
	 *	Returns an element by its index
	 *
	 * Parameters:
	 *
	 *	index
	 */
	p.getElementByIndex = function (index)
	{
		if (typeof(index) == 'object')
		{
			if (!index.length) return [];
			
			var i, result = new Array();
			for (i=0; i<index.length; i++)
			{
				result.push(this.indexed[index[i]]);
			}

			return result;
		}
		else return this.indexed[index];
	}

	/**
	 * Method: i
	 *
	 *	Returns an element by its index (alias of getElementByIndex)
	 */
	p.i = p.getElementByIndex;

	/**
	 * Method: getElementByName
	 *
	 *	Returns an element by its name
	 *
	 * Parameters:
	 *
	 *	name 
	 */
	p.getElementByName = function (name)
	{
		if (typeof(name) == 'object')
		{
			if (!name.length) return [];

			var i, result = new Array();
			for (i=0; i<name.length; i++)
			{
				result.push(this.named[name[i]]);
			}

			return result;
		}
		return this.named[name];
	}

	/**
	 * Method: n
	 *
	 *	Returns an element by its name (alias of getElementByName)
	 */
	p.n = p.getElementByName;

	/**
	 * Method: getNameByIndex
	 *
	 *	Gets the name of an element, given its index
	 *
	 * Parameters:
	 *
	 *	index - 
	 */
	p.getNameByIndex = function(index)
	{
		var elm = this.getElementByIndex(index);
		return this.getElementName(elm);
	}

	/**
	 * Method: name2index
	 *
	 *	Gets the name of an element, given its index (alias of getNameByIndex)
	 *
	 * Parameters:
	 *
	 *	index - 
	 */
	p.name2index = p.getIndexByName;

	/**
	 * Method: getIndexByName
	 *
	 *	Gets the index of an element, given its name
	 *
	 * Parameters:
	 *
	 *	name - 
	 */
	p.getIndexByName = function(name)
	{
		var elm = this.getElementByName(name);
		return this.getElementIndex(elm);
	}

	/**
	 * Method: index2name
	 *
	 *	Gets the index of an element, given its name (alias of getIndexByName)
	 *
	 * Parameters:
	 *
	 *	name - 
	 */
	p.index2name = p.getNameByIndex;

	/*************************************************************************\
	 *                        Group: Index and name changing                 *
	\*************************************************************************/

	/**
	 * Method: changeElementName
	 *
	 *	Changes the name of an element
	 *
	 * Parameters:
	 *
	 *	info - object with the following indexes:
	 *		oldName - this or elm or index are obligatory
	 *		elm - this or oldName or index are obligatory
	 *		index - this or oldName or elm are obligatory
	 *		name - (obligatory)
	 */
	p.changeElementName = function (info)
	{
		if (typeof(info) != 'object' ||
		   (info.oldName == null && info.elm == null && info.index == null) ||
		   info.name == null) throw({result: 'Invalid Parameters', location: 'thyCollection.changeElementName'});
		   
		var found = false;
		
		if (info.elm == null && info.oldName == null && 
		   (info.elm = this.indexed[info.index]) != null && 
		   (info.oldName == this.getElementName(info.elm)) != null) found = true;
		   
		else if (info.elm == null && info.index == null && 
		        (info.elm = this.named[info.oldName]) != null &&
		        (info.index = this.getElementIndex(info.elm)) != null) found = true;

		else if (info.elm != null && info.oldName == null &&
		        (info.oldName = this.getElementName(info.elm)) != null &&
		        (info.index != null || (info.index = this.getElementIndex(info.elm) != null))) found = true;

		else if (info.elm != null && info.index == null &&
		        (info.index = this.getElementIndex(info.elm)) != null &&
				(info.oldName != null || (info.oldName = this.getElementName(info.elm) != null))) found = true;

		if (!found || info.oldName == info.name) return;

		var existing = this.getElementByName(info.name);

		if (existing != null)
		{
			this._swapNames(info.elm, info.oldName, existing, info.name);
		}
		else
		{
			this.named[info.name] = info.elm;
			this.named[info.oldName] = '';
			delete this.named[info.oldName];
		}
	}

	/**
	 * Method: changeElementIndex
	 *
	 *	Changes the index of an element
	 *
	 * Parameters:
	 *
	 *	info - object with the following indexes:
	 *		oldName - this or elm or index are obligatory
	 *		elm - this or oldName or index are obligatory
	 *		index - this or oldName or elm are obligatory
	 *		name - (obligatory)
	 */
	p.changeElementIndex = function (info, name_also)
	{
		if (typeof(info) != 'object' || 
		   (info.name == null && info.elm == null && info.oldIndex == null) || 
		   info.index == null) return;
		   
		if (info.index >= this.indexed.length) info.index = this.indexed.length-1;

		var found = false;
		if (info.elm == null && info.oldIndex == null) { info.elm = this.named[info.name]; found = true; }
		else if (info.elm == null && info.name == null) { info.elm = this.indexed[info.oldIndex]; found = true; }
		else if (info.elm != null) found = true;

		var index = info.oldIndex || this.getElementIndex(info.elm);
		var name = info.name || this.getElementName(info.elm);
		if (!found || index == null) return;
		if (index == info.index) return index;
		if (name_also)
		{
			var name_other = this.getElementName(this.indexed[info.index]);
			this._swapNames(info.elm, name, this.indexed[info.index], name_other);
		}

		var start, end;

		if (index < info.index)
		{
			this.remove(info.elm, name, index);
			start = this.indexed.slice(0, info.index);
			end   = this.indexed.slice(info.index);
		}
		else
		{
			start = this.indexed.slice(0, info.index);
			this.remove(info.elm, name, index);
			end   = this.indexed.slice(info.index);
		}
		this.indexed = start.concat(info.elm, end);
		this.named[name] = info.elm;
		return info.index;
	}

	/*************************************************************************\
	 *                        Group: Swapping                                *
	\*************************************************************************/

	/**
	 * Method: swapElements
	 *
	 *	Swaps the names and indexes of two elements
	 *
	 * Parameters:
	 *
	 *	elm1
	 *	elm2
	 */
	p.swapElements = function (elm1, elm2)
	{
		if (!this.swapNames(elm1, elm2) || !this.swapIndex(elm1, elm2)) return false;
	}

	/**
	 * Method: swapNames
	 *
	 *	Swaps the names of two elements
	 *
	 * Parameters:
	 *
	 *	elm1
	 *	elm2
	 */
	p.swapNames = function (elm1, elm2)
	{
		var name1, name2;
		if (!(name1 = this.getElementName(elm1)) || !(name2 = this.getElementName(elm2))) return false;

		this._swapNames(elm1, name1, elm2, name2);
		return true;
	}

	/**
	 * Method: swapIndex
	 *
	 *	Swaps the indexes of two elements
	 *
	 * Parameters:
	 *
	 *	elm1
	 *	elm2
	 */
	p.swapIndex = function (elm1, elm2)
	{
		var index1, index2;
		if (!(index1 = this.getElementIndex(elm1)) || !(index2 = this.getElementIndex(elm2))) return false;

		this._swapIndex(elm1, index1, elm2, index2);
		return true;
	}

	/*************************************************************************\
	 *                        Group: Miscelaneous                            *
	\*************************************************************************/

	/**
	 * Method: toString
	 *
	 *	Returns a string representation of the collection
	 *
	 */
	p.toString = function ()
	{
		var i, str = '';
		var name;

		for (i=0; i<this.indexed.length; i++)
		{
			name = this.getElementName(this.indexed[i]);
			str += 'i: '+i+', value: '+this.indexed[i].toString()+'\n';
			str += 'n: '+name+', value: '+this.named[name].toString()+'\n';
		}

		return str;
	}

	

	p._swapNames = function (elm1, name1, elm2, name2)
	{
		this.named[name1] = elm2;
		this.named[name2] = elm1;
	}

	p._swapIndex = function (elm1, index1, elm2, index2)
	{
		this.indexed[index1] = elm2;
		this.indexed[index2] = elm1;
	}


