 /*****************************************************************************\
 * ThyAPI - Thyamad Javascript API - Client<->Server Connection Element       *
 * http://www.thyamad.com                                                     *
 *                                                                            *
 * Copyright (C) 2005 - Raphael Derosso Pereira                               *
 * Based on DynAPI v3.0b1                                                     *
 * -------------------------------------------------------------------------  *
 *  This library is free software; you can redistribute it and/or modify it   *
 *  under the terms of the GNU Lesser General Public License as published     *
 *  by the Free Software Foundation; either version 2.1 of the License,       *
 *  or any later version.                                                     *
 *  This library is distributed in the hope that it will be useful, but       *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of                *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
 *  See the GNU Lesser General Public License for more details.               *
 \****************************************************************************/

	/**
	 * Class: thyConnector
	 *
	 *	Class that handles raw server connections. It handles HTTP requests to
	 *	server and server responses.
	 *
	 *	It is also responsible to inform user about connection status.
	 *
	 */
	function thyConnector()
	{
		this.thyBase = thyBase;
		this.thyBase();

		/* Public Attributes */
		this.requests = new Array();
		this.progressContents = new Array();
		this.visible = false;

		var _this = this;
		this.nReqs = 0;

		/* Private Attributes */
		this._progressBox = null;
		this._progressHolder = {};

		// Initialization \\
		this.progressContents[0] = {};
		this.progressContents[1] = {};
		this.progressContents[2] = {};
		this.progressContents[3] = {};

	}

	p = dynapi.setPrototype('thyConnector','thyBase');

	/**
	 * Method: newRequest
	 *
	 *	Opens up a HTTP channel to target, sending data using GET or
	 *	POST methods and sets up a handler to manipulate server response
	 *
	 * Parameters:
	 *
	 *	id      - The ID of the request. This is used to avoid double calls and
	 *	          show the appropriate message in the appropriate place
	 *
	 *	target  - The target URL
	 *
	 *	data    - The string to be sent
	 *
	 *	handler - A function that will be called when server response arrives.
	 *	          This function will be called with the response from server passed
	 *	          as argument.
	 *
	 *	method  - How data should be passed to server. Valid values are 'POST' and 'GET'.
	 *	          Defaults to POST
	 *
	 */
	p.newRequest = function (id, target, data, handler, method)
	{
		var _this = this;

		if (this.requests[id] && this.requests[id] != null)
		{
			// TODO: Study better ways to do this...
			return;
		}

		// Change cursor to progress state \\
		dynapi.document.doc.body.style.cursor = 'progress';
		
		this.nReqs++;
		
		var oxmlhttp = null;
		
		try
		{ 
			oxmlhttp = new XMLHttpRequest();
			oxmlhttp.overrideMimeType('text/xml');
		}
		catch (e)
		{ 
			try
			{
				oxmlhttp = new ActiveXObject('Msxml2.XMLHTTP');
			}
			catch (e1)
			{ 
				try
				{
					oxmlhttp = new ActiveXObject('Microsoft.XMLHTTP');
				}
				catch (e2)
				{
					oxmlhttp = null;
				}
			}
		}
		
		if (!oxmlhttp)
		{
			throw({result: 'unsuported browser', location: 'thyConnector.newRequest'});
		}
		
		this.requests[id] = oxmlhttp;

		var _this = this;
		
		var treatResponse = function ()
		{
			if (oxmlhttp.readyState == 4 )//&& oxmlhttp.channel.status == 0)
			{
				dynapi.debug.print('\nServer Message Arrived:\n\n'+oxmlhttp.responseText+'\n\n');
				switch (oxmlhttp.status)
				{
					case 200:
						if (typeof(handler) == 'function')
						{
							if (oxmlhttp.responseXML) handler(oxmlhttp.responseXML);
							else handler(oxmlhttp.responseText);
							//handler(oxmlhttp.responseText);
						}
						delete _this.requests[id];
						_this.nReqs--;
						if (!_this.nReqs) dynapi.document.doc.body.style.cursor = '';
						break;

					case 404:
						alert('Page Not Found!');
						break;

					default:
						//alert('Some problem while accessing the server. The status is '+oxmlhttp.status);
				}
			}
		}
		
		var sub_handler = function ()
		{
			try
			{
				_this._setProgressState(id, oxmlhttp.readyState);
				
				treatResponse();
			}
			catch (e)
			{
				_this.nReqs--;
				if (!_this.nReqs) dynapi.document.doc.body.style.cursor = '';
				_this._setProgressState(id, 4);
				delete _this.requests[id];

				// TODO: Handle of special exceptions must be made by API
				
				if (e && typeof(e) == 'object' && e.result && e.location)
				{
					showMessage(e.result+' in '+e.location);
					return;
				}
				
				// Exception 0x80040111 => trying to access a component without permission to do so
				if (e && typeof(e) == 'object' && e.result && e.result == 0x80040111)
				{
					return;
				}

				showMessage(e);
			}
		}

		try
		{ 
			if (method == 'GET')
			{
				dynapi.debug.print('\nSending Message to Server at '+target+':\n\n'+data+'\n\n');
				oxmlhttp.onreadystatechange = sub_handler;
				oxmlhttp.open("GET",target,true);
				oxmlhttp.send(null);
			}
			else if (method == 'POST' || !method)
			{
				dynapi.debug.print('\nSending Message to Server at '+target+':\n\n'+data+'\n\n');
				oxmlhttp.onreadystatechange = sub_handler;
				oxmlhttp.open("POST",target, true);
				oxmlhttp.setRequestHeader('Content-Type',this.contentType);
				oxmlhttp.send(data);
			}
		}
		catch(e)
		{ 
			showMessage(e);
		}
		
		return true;
	}

	/**
	 * Method: setContentType
	 *
	 *	Sets up the Content-type HTTP header attribute that will be sent to server
	 *	when a new request is made
	 *
	 * Parameter:
	 *
	 *	type - The Content-type string
	 *
	 */
	p.setContentType = function(type)
	{
		this.contentType = type;
	}

	/**
	 * Method: cancelRequest
	 *
	 *	Cancel the request specified by ID
	 *
	 * Parameter:
	 *
	 *	id - The request creation ID
	 *
	 */
	p.cancelRequest = function (id)
	{
		if (!this.requests[id])
		{
			return false;
		}

		this.requests[id].abort();
	}

	/**
	 * Method: setProgressContent
	 *
	 *	When creating a request, one can specify what message should be shown
	 *	on the various possible states of the connection. This method is used
	 *	to do so.
	 *
	 * Parameters:
	 *
	 *	id      - The ID of the request
	 *
	 *	state   - The state that this message should be shown
	 *
	 *	content - The content of the message
	 *
	 */
	p.setProgressContent = function (id, state, content)
	{
		switch (state)
		{
			case 0:
			case 'UNINITIALIZED':
				this.progressContents[0][id] = content;
				break;

			case 1:
			case 'LOADING':
				this.progressContents[1][id] = content;
				break;

			case 2:
			case 'LOADED':
				this.progressContents[2][id] = content;
				break;

			case 3:
			case 'INTERACTIVE':
				this.progressContents[3][id] = content;
				break;

			case 4:
			case 'COMPLETED':
				this.progressContents[4][id] = content;
				break;

			default:
				throw('INVALID STATE!');
		}
	}

	/**
	 * Method: setProgressHolder
	 *
	 *	Specifies a location in the document where the status messages should
	 *	be shown.
	 *
	 * Parameters:
	 *
	 *	id     - The request ID
	 *
	 *	holder - The holder. It can be a thyAPI component, a DOM element ID or
	 *	         a DOM element itself
	 *
	 */
	p.setProgressHolder = function (id, holder)
	{
		var objHolder;
		
		if (typeof(holder) == 'object' && holder.DynLayer)
		{
			this._progressHolder[id] = holder;
		}
		else if (typeof(holder) == 'string')
		{
			var obj = dynapi.document.getElementByID(holder);
			this._progressHolder[id] = new DynLayer();
			this._progressHolder[id].setAnchor({topA: obj, leftA: obj});
		}
		else if (typeof(holder) == 'object')
		{
			this._progressHolder[id] = new DynLayer();
			this._progressHolder[id].setAnchor({topA: holder, leftA: holder});
		}
		else
		{
			return false;
		}

		this._progressHolder[id].setVisible(false);
	}

	/**
	 * Method: setVisible
	 *
	 *	Sets the visibility of the messages. If true, messages are shown.
	 *	Else, messages are not shown.
	 *
	 * Parameter:
	 *
	 *	visible - true/false
	 *
	 */
	p.setVisible = function (visible)
	{
		this.visible = (visible) ? true : false;
		if (!visible)
		{
			for (i in this._progressHolder)
			{
				this._progressHolder[i].setVisible(visible);
			}
		}
	}

	/****************************************************************************\
	 *                         Group: Private Methods                           *
	\****************************************************************************/
	
	/**
	 * Method: _setProgressState
	 *
	 *	Internal method used to update status messages
	 *
	 * Parameters:
	 *
	 *	id - The request ID
	 *
	 *	state - The actual state of the connection
	 *
	 */
	p._setProgressState = function (id, state)
	{
		switch (state)
		{
			case 0:
			case 4:
				if (!this._progressHolder[id]) return;
				this._progressHolder[id].setVisible(false);
				break;

			default:
				if (!this.visible)
				{
					return;
				}
				
				if (!this._progressHolder[id]) return;

				this._progressHolder[id].setHTML(this.progressContents[state][id] || '');
				this._progressHolder[id].setVisible(true);
		}
	}

	var thyConnectorObj = new thyConnector();
	thyConnectorObj.setVisible(true);
