 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Collection Element                      *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/


	/**
	 * Class: thyVisualCollection
	 *
	 *	This is the visual ThyAPI Collection Element. With it, one can hold a 
	 *	collection of any visual Element (that is, any element derivate from
	 *	<thyPanel>) and it will perform the <thyCollection> actions.
	 *
	 * Parameters:
	 *
	 *	name - The element name
	 *	vParent - The parent element (*must* be derivate from <thyPanel>)
	 *
	 */
	function thyVisualCollection(name, vParent)
	{
		this.thyCollection = thyCollection;
		this.thyCollection(name);
		
		this.vParent = vParent;
		this.defaultName = 'VisualElement';
	}

	p = dynapi.setPrototype('thyVisualCollection', 'thyCollection');

	p._thyCollectionPopulate = p.populate;
	/**
	 * Method: populate
	 *
	 *	Populate the <thyVisualCollection> with the passed arguments.
	 *
	 *	- *Note* - The arguments *must* be visual elements!
	 *
	 * Parameter:
	 *
	 *	data - An array of visual elements
	 *
	 */
	p.populate = function (data)
	{
		if (data == null) throw({result: 'Invalid Data', location: 'thyVisualCollection.populate'});

		if (typeof(data) != 'object' || !data.length) data = [data];
		
		var i;
		for (i=0; i<data.length; i++)
		{
			if (typeof(data[i]) != 'object' || !data[i].isThyPanel)
				throw({result: 'Invalid Data', location: 'thyVisualCollection.populate'});
		}
		
		for (i=0; i<data.length; i++)
		{
			this._thyCollectionPopulate(data[i]);
		}
	}

	p._thyCollectionAdd = p.add;
	/**
	 * Method: add
	 *
	 *	Inserts a visualElement into the collection and add it as child of vParent
	 *
	 * Parameters:
	 *
	 *	elm - The visual element
	 *	name - The name of the element
	 *	index - The index position this element should have.
	 *
	 */
	p.add = function (elm, name, index)
	{
		if (!elm || typeof(elm) != 'object' || !elm.isThyPanel) throw({result: 'Invalid Visual Element', location: 'thyVisualCollection.add'});

		var iIndex;
		if ((iIndex = this._thyCollectionAdd(elm, name)) == null) return;

		this.vParent.addChild(elm);

		if (index != null)
		{
			return this.changeElementIndex({elm: elm, oldIndex: iIndex,index: index});
		}

		return this.indexed.length-1;
	}

	p._thyCollectionRemove = p.remove;
	/**
	 * Method: remove
	 *
	 *	Removes the element from vParent and from collection
	 *
	 * Parameters:
	 *
	 *	elm - The element itself
	 *	name - The element name (optional)
	 *	index - The element index (optional)
	 *
	 */
	p.remove = function (elm, name, index)
	{
		if (elm == null) return;
		if (name == null && (name = this.getElementName(elm)) == null) return;
		if (index == null && (index = this.getElementIndex(elm)) == null) return;
		
		this.vParent.removeChild(elm);
		
		this._thyCollectionRemove(elm, name, index);
	}

	p._thyCollectionChangeElementIndex = p.changeElementIndex;
	/**
	 * Method: changeElementIndex
	 *
	 *	Change the position of an element inside the collection and
	 *	on vParent
	 *
	 * Parameters:
	 *
	 *	info - Array that may specify _elm_, _name_ or _oldIndex_ and new _index_
	 *	name_also - If true, the name of the element will be exchanged with the 
	 *	            name of the element in the destination index.
	 *
	 */
	p.changeElementIndex = function (info, name_also)
	{
		var index = this._thyCollectionChangeElementIndex(info, name_also);
		
		if (index == null) return;

		if (index == info.oldIndex) return index;

		this.vParent.removeChild(info.elm);
		this.vParent.addChild(info.elm, index);

		return index;
	}

	p._thyCollection_swapIndex = p.swapIndex;
	/**
	 * Method: _swapIndex
	 *
	 *	Exchange the position of 2 elements
	 *
	 * Parameters:
	 *
	 *	elm1 - The first element
	 *	index1 - The position of the first element
	 *	elm2 - The second element
	 *	index2 - The position of the second element
	 *
	 */
	p._swapIndex = function (elm1, index1, elm2, index2)
	{
		this._thyCollection_swapIndex(elm1, index1, elm2, index2);

		this.vParent.removeChild(elm1);
		this.vParent.removeChild(elm2);

		if (index1 < index2)
		{
			this.vParent.addChild(elm2, index1);
			this.vParent.addChild(elm1, index2);
		}
		else if (index1 > index2)
		{
			this.vParent.addChild(elm1, index2);
			this.vParent.addChild(elm2, index1);
		}
	}
