 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Border Panel Element                    *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/

	/**
	 * Class: thyBorderPanel
	 *
	 *	Base class for all other ThyAPI GUI Components that wants to have custom
	 *	(not CSS) borders.
	 *
	 *	This class is identical to thyPanel class, except that it creates 5 
	 *	internal layers: 4 borders and the content holder.
	 *
	 * CSS Classes for Common Attributes:
	 *
	 *	.thyBorderPanel_contents - The class of the the contents panel
	 *	.thyBorderPanel_borders  - Common class all borders inherits
	 *	.thyBorderPanel_hBorders - Common class for top and bottom borders
	 *	.thyBorderPanel_vBorders - Common class for left and right borders
	 *	.thyBorderPanel_topBorder - The class for the top border
	 *	.thyBorderPanel_bottomBorder - The class for the bottom border
	 *	.thyBorderPanel_rightBorder - The class for the right border
	 *	.thyBorderPanel_leftBorder - The class for the left border
	 *
	 * CSS Classes for Specific Attributes:
	 *
	 *	- .<object name>_contents
	 *	- .<object name>_borders
	 *	- .<object name>_hBorders
	 *	- .<object name>_vBorders
	 *	- .<object name>_topBorder
	 *	- .<object name>_bottomBorder
	 *	- .<object name>_rightBorder
	 *	- .<object name>_leftBorder
	 *
	 * Author:
	 *
	 *	Raphael Derosso Pereira <raphaelpereira@users.sourceforge.net>
	 *
	 *
	 */
	function thyBorderPanel(name,html)
	{
		this.thyPanel = thyPanel;
		this.thyPanel(name,html);

		// Attributes
		this._contents = {
			main : null,
			t : null,
			b : null,
			l : null,
			r : null
		};
		this.cssClasses['contents'] = ['contents'];
		this.cssClasses['top']     = ['borders','hBorders','topBorder'];
		this.cssClasses['bottom']  = ['borders','hBorders','bottomBorder'];
		this.cssClasses['left']    = ['borders','vBorders','leftBorder'];
		this.cssClasses['right']   = ['borders','vBorders','rightBorder'];
		this.addCSSClass('thyBorderPanel');
	}
	
	p = dynapi.setPrototype('thyBorderPanel','thyPanel');

	p._thyPanelGetInnerHTML = p.getInnerHTML;
	/**
	 * Method: getInnerHTML
	 *
	 * Overwritten method that insert borders on result
	 *
	 */
	p.getInnerHTML = function()
	{
		var html=['<table id="'+this.name+'_mainHolder'+'" cellspacing="0" cellpadding="0" border="0">',
		       '<tr><td id="'+this.name+'_t'+'" class="'+this.getCSSClasses('top')+'" colspan="3"></td></tr>',
		       '<tr><td id="'+this.name+'_l'+'" class="'+this.getCSSClasses('left')+'"></td>',
		       '<td id="'+this.name+'_contentsHolder" class="'+this.getCSSClasses('contents')+'">',
			   '<div style="position: relative;">'+this._thyPanelGetInnerHTML()+'</div>',
			   '</td>',
		       '<td id="'+this.name+'_r'+'" class="'+this.getCSSClasses('right')+'"></td></tr>',
		       '<tr><td id="'+this.name+'_b'+'" class="'+this.getCSSClasses('bottom')+'" colspan="3"></td></tr></table>'].join('');
		//alert(html);
		return html;
	} 

	/*************************************************************************\
	 *                          Private Methods                              *
	\*************************************************************************/
	
	p._thyPanelAssignElement = p._assignElement;
	/**
	 * Method: _assignElement
	 *
	 * Overwritten method that assigns borders too
	 *
	 */
	p._assignElement = function (dlyr,elm,divs)
	{
		this._assignBorders(dlyr);
		this._thyPanelAssignElement(dlyr,elm,divs);
	}

	p._thyPanelAssignElm = p._assignElm;
	/**
	 * Method: _assignElm
	 *
	 * Overwritten method that assigns borders too
	 *
	 */
	p._assignElm = function(elm)
	{
		if (this._contents.main &&
		    this._contents.t &&
			this._contents.b &&
			this._contents.l &&
			this._contents.r) return true;

		this._assignBorders(this);
		this._thyPanelAssignElm(elm);
	}

	/**
	 * Method: _assignBorders
	 *
	 * Associates the HTML border elements to internal variables
	 *
	 */
	p._assignBorders = function(dlyr)
	{
		dlyr._contents.holder = dynapi.functions.getElementById(this.name+'_contentsHolder');
		dlyr._contents.t = dynapi.functions.getElementById(this.name+'_t');
		dlyr._contents.l = dynapi.functions.getElementById(this.name+'_l');
		dlyr._contents.r = dynapi.functions.getElementById(this.name+'_r');
		dlyr._contents.b = dynapi.functions.getElementById(this.name+'_b');
	}
