 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Dialog Window Element                   *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/

	/**
	 * Class: thyDialogWindow
	 *
	 * This class implements the Dialog Window widget.
	 *
	 * CSS Classes:
	 *
	 *	- <class name>_buttons
	 *
	 *	- <object name>_buttons
	 *
	 * Events:
	 *
	 *	- onsave - Called when button with code 0 is clickd
	 *	- oncancel - Called when button with code 1 is clicked
	 *	- oncode<n> - Called when button with code <n> is clicked
	 */
	function thyDialogWindow(name,title,html)
	{
		this.thyWindow = thyWindow;
		this.thyWindow(name,title,html);

		// Attributes \\
		this.buttons = [];
		this.buttonLayer = new thyPanel(this.name+'_buttons');
		this.addCSSClass('thyDialogWindow');

		this._thyBorderPanelAddChild(this.buttonLayer);

		//if (!this.thyDialogWindow) this.onPreCreate(this.initThyDialogWindow);
		this.onPreCreate(this.initThyDialogWindow);
	}

	p = dynapi.setPrototype('thyDialogWindow','thyWindow');

	/**
	 * Method: initThyDialogWindow
	 *
	 *	Initialization method
	 *
	 */
	p.initThyDialogWindow = function ()
	{
	}
	
	/**
	 * Method: setButtons
	 *
	 *	Tells the Window which buttons it should have
	 *
	 * Parameters:
	 *
	 *	buttons - Array that holds buttons infomations. It format is:
	 *
	 *		- buttons[n]['caption'] - The button caption
	 *		- buttons[n]['code'] - The button code, that can be:
	 *			- 0 - Save
	 *			- 1 - Cancel
	 *			- n - Other code
	 *
	 */
	p.setButtons = function (buttons)
	{
		if (!buttons || typeof(buttons) != 'object' || !buttons.length)
		{
			throw({result: 'Invalid Argument', location: 'thyDialogWindow.setButtons'});
		}

		var i, btn, nBtns = buttons.length;
		
		for (i=0; i<buttons.length; i++)
		{
			btn = new thyButton(this.name+'_button'+this.buttons.length, buttons[i]['caption'])
			btn.addCSSClass('thyDialogWindow_button');
			btn.addCSSClass('thyDialogWindow_button'+this.buttons.length);
			btn.setPosition('relative');
			btn.setFloat('right');
			btn.setX(0);
			btn.setY(0);
			btn.addEventListener({onclick: this._getButtonEvent(buttons[i])});
			this.buttons.push(btn);
			this.buttonLayer.addChild(btn);
		}
	}

	p._thyWindowSetName = p.setName;
	/**
	 * Method: setName
	 *
	 *	Overwritten method that sets the name of main and dialog buttons
	 *	and refreshes CSS Classes
	 *
	 * Parameter:
	 *
	 *	name - The new widget name
	 *
	 */
	p.setName = function (name)
	{
		if (name == null || this.name == name) return;
		
		this._thyWindowSetName(name);
		if (this.buttonLayer) this.buttonLayer.setName(name+'_buttons');
	}

	p._thyWindowSetCSSClass = p.setCSSClass;
	/**
	 * Method: setCSSClass
	 *
	 *	Overwritten method that calls parent setCSSClass and sets the class of
	 *	the button layer
	 *
	 * Parameter:
	 *
	 *	type - The class type
	 *
	 */
	p.setCSSClass = function (type)
	{
		this._thyWindowSetCSSClass(type);
		if (this.buttonLayer) this.buttonLayer.setCSSClass(type);
	}

	/*************************************************************************\
	 *                          Private Methods                              *
	\*************************************************************************/

	p._thyWindow_addCSSClass = p._addCSSClass;
	/**
	 * Method: _addCSSClass
	 *
	 *	Overwritten method that sets the class of dialog buttons also
	 *
	 */
	p._addCSSClass = function (cssClass)
	{
		this._thyWindow_addCSSClass(cssClass);
		if (this.buttonLayer) this.buttonLayer._addCSSClass(cssClass+'_buttons');
	}
	
	p._thyWindow_removeCSSClass = p._removeCSSClass;
	/**
	 * Method: _removeCSSClass
	 *
	 *	Overwritten method that removes the specified CSS Class from main and
	 *	dialog buttons
	 *
	 * Parameter:
	 *
	 *	cssClass - The CSS Class to be removed
	 *
	 */
	p._removeCSSClass = function (cssClass)
	{
		this._thyWindow_removeCSSClass(cssClass);
		if (this.buttonLayer) this.buttonLayer._removeCSSClass(cssClass+'_buttons');
	}

	/*************************************************************************\
	 *                               Events                                  *
	\*************************************************************************/

	/**
	 * Method: _getButtonEvent
	 *
	 *	Returns the event that is called on Dialog window buttons click.
	 *
	 *	The event calls the correct modal event
	 *
	 */
	p._getButtonEvent = function(button)
	{
		var self = this;
		
		return function (e)
		{
			switch (button['code'])
			{
				case 0: // Save
					self.invokeEvent('save');
					self.invokeEvent('code0');
					break;

				case 1: // Cancel
					self.invokeEvent('cancel');
					self.invokeEvent('code1');
					break;

				default:
					self.invokeEvent('code'+button['code']);
			}
		}
	}
