 /****************************************************************************\
 * ThyAPI - Thyamad Javascript API - Edit Box Element                        *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/
 	
	/**
	 * Class: thyEditBox
	 *
	 *	This is the edit box to be used in ThyAPI. In the present state, it is
	 *	just a label and a simple HTML input wrapped around a ThyAPI compatible 
	 *	system.
	 *
	 * CSS Classes:
	 *
	 *	- .thyEditBox_label
	 *	- .thyEditBox_editbox
	 *
	 *	- .<object name>_label
	 *	- .<object name>_contents
	 *
	 */
	function thyEditBox(name,label,value)
	{
		this.thyLabelPanel = thyLabelPanel;
		this.thyLabelPanel(name,label);

		// Attributes
		this._value = value || '';
		this.addCSSClass('thyEditBox');

		var self = this;
		this.onPreCreate(this.preInitThyEditBox);
		this.onCreate(this.initThyEditBox);	
	}
	
	p = dynapi.setPrototype('thyEditBox','thyLabelPanel');

	/**
	 * Method: initThyEditBox
	 *
	 *	Initialization Method
	 *
	 */
	p.initThyEditBox = function()
	{
		var self = this;
		
		this.contents.elm.childNodes[0]._dynobj = this;
	}

	/**
	 * Method: preInitThyEditBox
	 *
	 *	Initialization method executed right before element creation
	 *
	 */
	p.preInitThyEditBox = function()
	{
		this.contents.setRelativeness(false);
		this.setHTML('<input '+
			'class="thyEditBox_input '+this.name+'_input" '+
			'id="'+this.name+'_input" type="text" style="'+
			'width: 100%;'+
			'height: 100%;'+
			'left: 0px;'+
			'right: 0px'+
			'background: inherit;'+
			'background-color: inherit;'+
			'text-align: inherit;'+
			'color: inherit;'+
			'border: 0px;'+
			'z-index: inherit" '+
			(this.readOnly ? 'readonly ': '')+
			(this.disabled ? 'disabled ': '')+
			'value="'+this._value+'" />'
		);
	}
	
	/**
	 * Method: focus
	 *
	 *	Give the Edit Box Focus
	 *
	 */
	p.focus = function ()
	{
		if (!this._created) return;
		this.contents.elm.childNodes[0].focus();
	}

	/**
	 * Method: cleanUp
	 *
	 *	Resets the contents
	 *
	 */
	p.cleanUp = function ()
	{
		this.populate('');
	}
	
	/**
	 * Method: reset
	 *
	 *	Resets the contents
	 *
	 */
	p.reset = p.cleanUp;

	/**
	 * Method: getKeyEventElement
	 *
	 *	Returns the element that should be used to listen key events
	 *
	 */
	p.getKeyEventElement = function ()
	{
		return this.contents.elm.childNodes[0];
	}

	/**
	 * Method: getHeight
	 *
	 *	Overloaded method that returns the height of the input field
	 *
	 */
	p.getHeight = function ()
	{
		if (!this._created) return null;

		return this.contents.elm.childNodes[0].clientHeight;
	}

	/**
	 * Method: getWidth
	 *
	 *	Overloaded method that returns the width of the input field
	 *
	 */
	p.getWidth = function ()
	{
		if (!this._created) return null;

		return this.contents.elm.childNodes[0].clientWidth;
	}

	/*************************************************************************\
	 *                          Private Methods                              *
	\*************************************************************************/

	/**
	 * Method: _setReadOnly
	 *
	 *	Turns the EditBox into ReadOnly mode
	 *
	 */
	p._setReadOnly = function ()
	{
		if (!this._created) return;

		if (this.readOnly) this.contents.elm.childNodes[0].setAttribute('readonly', 'true');
		else this.contents.elm.childNodes[0].removeAttribute('readonly');

		this.contents.elm.childNodes[0].readonly = this.readOnly;
	}

	/**
	 * Method: _cleanUp
	 *
	 *	Removes editbox contents
	 *
	 */
	p._cleanUp = function ()
	{
		this.populate('');
	}

	/**
	 * Method: _reset
	 *
	 *	Removes editbox contents
	 *
	 */
	p._reset = p._cleanUp;
	
	p._thyLabelPanel_setDisabled = p._setDisabled;
	/**
	 * Method: _setDisabled
	 *
	 *	Executed when non-private <thyPanel> setDisabled method is called
	 *
	 */
	p._setDisabled = function ()
	{
		this._thyLabelPanel_setDisabled();

		if (!this._created) return;
		this.contents.elm.childNodes[0].setAttribute('disabled', '');
		this.contents.elm.childNodes[0].disabled = this.disabled;
	}

	/**
	 * Method: _populateContents
	 *
	 *	Update this element value
	 *
	 */
	p._populateContents = function(data)
	{
		this._value = data;
		
		if (!this._created) return;
		
		this.contents.elm.childNodes[0].value = data;
	}

	/**
	 * Method: _sweepOutContents
	 *
	 *	Returns the current value of this component
	 *
	 */
	p._sweepOutContents = function()
	{
		if (!this._created) return this._value;
		
		return this.contents.elm.childNodes[0].value;
	}


