 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Panel Element                           *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/

	/**
	 * Class: thyLabelPanel
	 *
	 *	This is a Panel that has a Label.
	 *
	 * CSS Classes:
	 *
	 *	- .thyLabelPanel_label
	 *	- .thyLabelPanel_contents
	 *
	 */
	function thyLabelPanel(name, label, contents)
	{
		this.thyPanel = thyPanel;
		this.thyPanel(name);

		//** Attributes **\\
		this.contents = new thyPanel(this.name+'_contents', contents);
		this.label = new thyPanel(this.name+'_label', label);

		this.addCSSClass('thyLabelPanel');

		this._thyPanelAddChild(this.label);
		this._thyPanelAddChild(this.contents);

		this.onPreCreate(this.initThyLabelPanel);
	}

	p = dynapi.setPrototype('thyLabelPanel', 'thyPanel');

	/**
	 * Method: initThyLabelPanel
	 *
	 *	Initialization Method
	 *
	 */
	p.initThyLabelPanel = function ()
	{
		// Label \\
		this.label.setRelativeness(false);

		this.setCSSClass();
	}

	/**
	 * Method: populate
	 *
	 *	Populates the Panel and optionally its Label
	 *
	 * Parameter:
	 *
	 *	data - The contents of the panel or and array which have in its
	 *	       first element the Label and second the contents
	 *
	 */
	p.populate = function (data)
	{
		if (data == null) return;

		switch (typeof(data))
		{
			case 'object':
				if (data['defaultValue'] && typeof(this.setDefaultValue) == 'function')
				{
					this.setDefaultValue(data['defaultValue']);

					if (data['label'] == null && data['contents'] == null) return;

					delete data['defaultValue'];
				}
					
				if (data['label'] != null)
				{
					this.setLabel(data['label']);
					if (data['contents'] == null) return;
				}
				
				if (data['contents'] != null) data = data['contents'];
				break;
		}
		
		if (this._populateContents) this._populateContents(data);
		else this.contents.populate(data);
	}

	/**
	 * Method: sweepOut
	 *
	 *	Returns the sweepOut of the contents
	 *
	 */
	p.sweepOut = function ()
	{
		if (this._sweepOutContents) return this._sweepOutContents();

		return this.contents.sweepOut();
	}

	/**
	 * Method: cleanUp
	 *
	 *	Removes all data from contents
	 *
	 */
	p.cleanUp = function ()
	{
		this.contents.cleanUp();
		this._cleanUp();
	}

	/**
	 * Method: reset
	 *
	 *	Removes all transient data from contents panel
	 *
	 */
	p.reset = function ()
	{
		this.contents.reset();
		this._reset();
	}
	
	/**
	 * Method: setLabel
	 *
	 *	Sets this Panel Label text
	 *
	 * Parameter:
	 *
	 *	label - The text label
	 *
	 */
	p.setLabel = function (label)
	{
		this.label.setHTML(label);
	}
	
	/**
	 * Method: getLabel
	 *
	 *	Returns the Panel label text
	 *
	 */
	p.getLabel = function ()
	{
		return this.label.getHTML();
	}

	p._thyPanelSetHTML = p.setHTML;
	/**
	 * Method: setHTML
	 *
	 *	Sets the HTML of the contents.
	 *
	 * Parameter:
	 *
	 *	html - The HTML to be set inside Contents
	 *
	 */
	p.setHTML = function (html)
	{
		this.contents.setHTML(html);
	}
	
	p._thyPanelSetName = p.setName;
	/**
	 * Method: setName
	 *
	 *	Overwritten method that sets the name of main, label and contents
	 *	and refreshes CSS Classes
	 *
	 * Parameter:
	 *
	 *	name - The new widget name
	 *
	 */
	p.setName = function (name)
	{
		if (name == null || this.name == name) return;
		
		this._thyPanelSetName(name);
		if (this.label) this.label.setName(name+'_label');
		if (this.contents) this.contents.setName(name+'_contents');
	}
	
	p._thyPanelSetCSSClass = p.setCSSClass;
	/**
	 * Method: setCSSClass
	 *
	 *	Overwritten method that sets CSS classes of main, label and
	 *	contents panels
	 *
	 */
	p.setCSSClass = function (type)
	{
		this._thyPanelSetCSSClass(type);
		if (this.label) this.label.setCSSClass(type);
		if (this.contents) this.contents.setCSSClass(type);
	}
	
	p._thyPanelAddChild = p.addChild;
	/**
	 * Method: addChild
	 *
	 *	Overloaded method that add childs to Contents Panel
	 *
	 * Parameter:
	 *
	 *	child - The child object
	 *	alias - ? (dynapi property)
	 *	inlineID - ? (dynapi property)
	 *
	 */
	p.addChild = function (child, alias, inlineID)
	{
		this.contents.addChild(child,alias,inlineID);
	}

	p._thyPanelRemoveChild = p.removeChild;
	/**
	 * Method: removeChild
	 *
	 *	Overloaded method that removes childs from Contents panel
	 *
	 * Parameter:
	 *
	 *	child - The child to be removed or its index
	 *
	 */
	p.removeChild = function (child)
	{
		this.contents.removeChild(child);
	}
	
	/*************************************************************************\
	 *                       Group: Private Methods                          *
	\*************************************************************************/

	p._thyPanel_addCSSClass = p._addCSSClass;
	/**
	 * Method: _addCSSClass
	 *
	 *	Adds the specified CSS Class to base classes and reset CSS
	 *
	 */
	p._addCSSClass = function (cssClass)
	{
		this._thyPanel_addCSSClass(cssClass);
		if (this.label) this.label._addCSSClass(cssClass+'_label');
		if (this.contents) this.contents._addCSSClass(cssClass+'_contents');
	}

	p._thyPanel_removeCSSClass = p._removeCSSClass;
	/**
	 * Method: _removeCSSClass
	 *
	 *	Removes the specified CSS Class from main, label and contents and
	 *	refreshes CSS
	 *
	 * Parameter:
	 *
	 *	cssClass - The CSS Class to be removed
	 *
	 */
	p._removeCSSClass = function (cssClass)
	{
		this._thyPanel_removeCSSClass(cssClass);
		if (this.label) this.label._removeCSSClass(cssClass+'_label');
		if (this.contents) this.contents._removeCSSClass(cssClass+'_contents');
	}

	/**
	 * Method: _setDisabled
	 *
	 *	Insert 'thyLabelPanel_disabled' CSS class onto contents
	 *
	 */
	p._setDisabled = function ()
	{
		if (this.disabled)
		{
			this.contents.addCSSClass('thyLabelPanel_disabled');
		}
		else
		{
			this.contents.removeCSSClass('thyLabelPanel_disabled');
		}
	}


