 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Text Editor Element                     *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/


	/**
	 * Class: thyTextEdit
	 *
	 *	This class provides a Text Area Element wrapped inside a thyAPI like
	 *	widget.
	 *
	 */
	function thyTextEdit(name, label, text)
	{
		this.thyLabelPanel = thyLabelPanel;
		this.thyLabelPanel(name, label);

		//** Attributes **\\
		this._value = text || '';
		this._txtarea = null;
		this.addCSSClass('thyTextEdit');

		//** Initialization **\\
		var self = this;
		this.onPreCreate(this.initThyTextEdit);

		this.onCreate(function()
		{
			if (dynapi.ua.ie)
			{
				setTimeout(function() 
				{
					self.contents.elm.childNodes[0].contentWindow.document.body.style.padding = '0';
					self.contents.elm.childNodes[0].contentWindow.document.body.style.margin = '0';
					self.contents.elm.childNodes[0].contentWindow.document.body.style.overflow = 'hidden';
					self.contents.elm.childNodes[0].contentWindow.document.body.style.width = '100%';
					self.contents.elm.childNodes[0].contentWindow.document.body.style.height = '100%';
					self.contents.elm.childNodes[0].contentWindow.document.body.innerHTML = '<textarea id="pagetext" style="width: 100%; height: 100%;padding: 0; margin: 0; border-width: 0; border-style: none"></textarea>';
					self._txtarea = self.contents.elm.childNodes[0].contentWindow.document.all['pagetext'];
				},100);
				return;
			}
			
			self._txtarea = self.contents.elm.childNodes[0];
		});
	}

	p = dynapi.setPrototype('thyTextEdit', 'thyLabelPanel');

	/**
	 * Method: initThyTextEdit
	 *
	 *	Initialization method
	 *
	 */
	p.initThyTextEdit = function()
	{
		this.contents.setRelativeness(false);

		if (dynapi.ua.ie)
		{
			this.setHTML('<iframe style="width: 100%; height: 100%"></iframe>');
			return;
		}
		
		this.setHTML('<textarea style="'+
		             'width: 100%;'+
					 'height: 100%;'+
					 'background: inherit;'+
					 'color: inherit;'+
					 'border: 0px;'+
					 'margin: 0px;'+
					 'z-index: inherit" '+
					 (this.readOnly ? 'readonly ': '')+
					 (this.disabled ? 'disabled ': '')+
					 '>'+
					 this._value+
					 '</textarea>');
	}

	p._getInnerHTML = p.getInnerHTML;
	p.getInnerHTML = function ()
	{
		//alert(this._getInnerHTML());
		return this._getInnerHTML();
	}
	
	/**
	 * Method: focus
	 *
	 *	Give the Edit Box Focus
	 *
	 */
	p.focus = function ()
	{
		if (!this._created) return;
		this._txtarea.focus();
	}

	/**
	 * Method: cleanUp
	 *
	 *	Resets the contents
	 *
	 */
	p.cleanUp = function ()
	{
		this.populate('');
	}
	
	/**
	 * Method: reset
	 *
	 *	Resets the contents
	 *
	 */
	p.reset = p.cleanUp;

	/**
	 * Method: getKeyEventElement
	 *
	 *	Returns the element that should be used to listen key events
	 *
	 */
	p.getKeyEventElement = function ()
	{
		this._txtarea._dynobj = this;
		return this._txtarea;
	}

	/**
	 * Method: _setReadOnly
	 *
	 *	Turns the EditBox into ReadOnly mode
	 *
	 */
	p._setReadOnly = function ()
	{
		if (!this._created) return;
		this._txtarea.setAttribute('readonly', this.readOnly);
		this._txtarea.readonly = this.readOnly;
	}

	/*************************************************************************\
	 *                          Private Methods                              *
	\*************************************************************************/

	/**
	 * Method: _cleanUp
	 *
	 *	Removes editbox contents
	 *
	 */
	p._cleanUp = function ()
	{
		this.populate('');
	}

	/**
	 * Method: _reset
	 *
	 *	Removes editbox contents
	 *
	 */
	p._reset = p._cleanUp;
	
	p._thyLabelPanel_setDisabled = p._setDisabled;
	/**
	 * Method: _setDisabled
	 *
	 *	Executed when non-private <thyPanel> setDisabled method is called
	 *
	 */
	p._setDisabled = function ()
	{
		this._thyLabelPanel_setDisabled();

		if (!this._created) return;
		this._txtarea.setAttribute('disabled', '');
		this._txtarea.disabled = this.disabled;
	}

	/**
	 * Method: _populateContents
	 *
	 *	Update this element value
	 *
	 */
	p._populateContents = function(data)
	{
		this._value = data;
		
		if (!this._created) return;
		
		this._txtarea.value = data;
	}

	/**
	 * Method: _sweepOutContents
	 *
	 *	Returns the current value of this component
	 *
	 */
	p._sweepOutContents = function()
	{
		if (!this._created) return this._value;
		
		return this._txtarea.value;
	}

